;; redline/theme.jl

;; based on minimal by Garcia Galego <rgarcia@eurielec.etsit.upm.es>
;; hacked up a bit (m3) by Jacob Berkman <jacob@helixcode.com>
;; hacked up some more by Jonas Linde <jonas@init.se>

;; (custom-set-frame-style 'default-frame-style 'redline)

(let* (
       
       (title-width
	(lambda (w)
	  (let
	      ((w-width (car (window-dimensions w))))
	    (max 0 (text-width (window-name w))))))
       
       (update-text-width
	(lambda (w)
	  (if (eq (window-get w 'current-frame-style) 'redline)
	      (rebuild-frame w))))
       
       (font (get-font "-misc-fixed-medium-r-normal-*-*-100-*-*-c-*-iso8859-1"))
       
       (font-colors (list (get-color "grey50") (get-color "white")))
       (colors      (list (get-color "black")  (get-color "black")))
       (pre-colors  (list (get-color "grey25") (get-color "red")))
       (post-colors (list (get-color "grey25") (get-color "red4")))

       (border-width 1)
       (-border-width -1)

(frame
 `(
   ;; Bar
   ((background . ,colors)
    (foreground . ,font-colors)
    (font . ,font)
    (text . ,window-name)
    (top-edge . 0)
    (height . 11)
    (right-edge . 0)
    (width . ,(lambda (w) (+ (title-width w) 12)))
    (x-justify . center)
    (y-justify . center)
    (class . title))

   ;; Window borders
   ((background . ,pre-colors)
    (top-edge . ,-border-width)
    (right-edge . ,-border-width)
    (left-edge . 0)
    (height . ,border-width)
    (class . title)
    )

   ((background . ,pre-colors)
    (top-edge . ,-border-width)
    (bottom-edge . ,-border-width)
    (left-edge . ,-border-width)
    (width . ,border-width)
    (class . title)
    )

   ((background . ,post-colors)
    (bottom-edge . ,-border-width)
    (left-edge . 0)
    (right-edge . ,-border-width)
    (height . ,border-width)
    (class . title)
    )

   ((background . ,post-colors)
    (top-edge . 0)
    (bottom-edge . 0)
    (right-edge . ,-border-width)
    (width . ,border-width)
    (class . title)
    )

   ;; Bar frame
   ((background . ,post-colors)
    (top-edge . 11)
    (right-edge . 0)
    (height . 1)
    (width . ,(lambda (w) (+ (title-width w) 12)))
    (class . title)
    )

   ((background . ,post-colors)
    (top-edge . -1)
    (right-edge . -1)
    (width . 1)
    (height . 13)
    (class . title)
    )

   ((background . ,pre-colors)
    (top-edge . -1)
    (right-edge . -1)
    (height . 1)
    (width . ,(lambda (w) (+ (title-width w) 14)))
    (class . title)
    )

   ((background . ,pre-colors)
    (top-edge . -1)
    (right-edge . ,(lambda (w) (+ (title-width w) 12)))
    (width . 1)
    (height . 13)
    (class . title)
    )
   )
 )

(shaped-frame
 `(
   ;; Bar
   ((background . ,colors)
    (foreground . ,font-colors)
    (font . ,font)
    (text . ,window-name)
    (top-edge . -12)
    (height . 11)
    (right-edge . 1)
    (width . ,(lambda (w) (+ (title-width w) 12)))
    (x-justify . center)
    (y-justify . center)
    (class . title))

   ;; Bar frame
   ((background . ,post-colors)
    (top-edge . -1)
    (right-edge . 1)
    (height . 1)
    (width . ,(lambda (w) (+ (title-width w) 12)))
    (class . title)
    )

   ((background . ,post-colors)
    (top-edge . -13)
    (right-edge . 0)
    (width . 1)
    (height . 13)
    (class . title)
    )

   ((background . ,pre-colors)
    (top-edge . -13)
    (right-edge . 0)
    (height . 1)
    (width . ,(lambda (w) (+ (title-width w) 14)))
    (class . title)
    )

   ((background . ,pre-colors)
    (top-edge . -13)
    (right-edge . ,(lambda (w) (+ (title-width w) 13)))
    (width . 1)
    (height . 13)
    (class . title)
    )
   )
 )

(transient-frame
 `(
   ;; Window borders
   ((background . ,pre-colors)
    (top-edge . ,-border-width)
    (right-edge . ,-border-width)
    (left-edge . 0)
    (height . ,border-width)
    (class . title)
    )

   ((background . ,pre-colors)
    (top-edge . ,-border-width)
    (bottom-edge . ,-border-width)
    (left-edge . ,-border-width)
    (width . ,border-width)
    (class . title)
    )

   ((background . ,post-colors)
    (bottom-edge . ,-border-width)
    (left-edge . 0)
    (right-edge . ,-border-width)
    (height . ,border-width)
    (class . title)
    )

   ((background . ,post-colors)
    (top-edge . 0)
    (bottom-edge . 0)
    (right-edge . ,-border-width)
    (width . ,border-width)
    (class . title)
    )
   )
 )

)


(add-frame-style 'redline
		 (lambda (w type)
		   (cond ((eq type 'shaped)
			  shaped-frame)
			 ((eq type 'transient)
			  transient-frame)
			 ((eq type 'shaped-transient)
			  shaped-frame)
			 ((eq type 'unframed)
			  nil-frame)
			 (t
			  frame))))

(call-after-property-changed 'WM_NAME update-text-width)
)
